@@ -7,12 +7,12 @@
     modify it under the terms of the GNU Library General Public
     License as published by the Free Software Foundation; either
     version 2 of the License, or (at your option) any later version.
-  
+
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     Library General Public License for more details.
-   
+
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
     the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
@@ -100,7 +100,7 @@ AudioSubSystem::AudioSubSystem()
 	d = new AudioSubSystemPrivate;
 	d->audioIO = 0;
 	d->audioIOInit = false;
-	
+
 	_running = false;
 	consumer = 0;
 	producer = 0;
@@ -109,6 +109,7 @@ AudioSubSystem::AudioSubSystem()
 
 AudioSubSystem::~AudioSubSystem()
 {
+	delete d->audioIO;
 	delete d;
 }
 
@@ -337,7 +338,7 @@ int AudioSubSystem::selectWriteFD()
 
 bool AudioSubSystem::check()
 {
-	bool ok = open();
+        bool ok = open();
 
 	if(ok) close();
 	return ok;
@@ -352,7 +353,7 @@ bool AudioSubSystem::open()
 	{
 		if(d->audioIOName == "")
 			_error = "couldn't auto detect which audio I/O method to use";
-		else	
+		else
 			_error = "unable to select '"+d->audioIOName+"' style audio I/O";
 		return false;
 	}
@@ -421,6 +422,7 @@ void AudioSubSystem::handleIO(int type)
 		/*
 		 * make sure that we have a fragment full of data at least
 		 */
+Rewrite:
 		while(wBuffer.size() < _fragmentSize)
 		{
 			long wbsz = wBuffer.size();
@@ -476,6 +478,9 @@ void AudioSubSystem::handleIO(int type)
 				}
 			}
 		}
+
+                // If we can write a fragment more, then do so right now:
+                if (space >= _fragmentSize*2) goto Rewrite;
 	}
 
 	assert((type & ioExcept) == 0);
@@ -591,3 +596,16 @@ void AudioSubSystem::adjustInputBuffer(int count)
 		count++;
 	}
 }
+
+void AudioSubSystem::emergencyCleanup()
+{
+	if(producer || consumer)
+	{
+		fprintf(stderr, "AudioSubSystem::emergencyCleanup\n");
+
+		if(producer)
+			detachProducer();
+		if(consumer)
+			detachConsumer();
+	}
+}
