@@ -221,6 +221,12 @@ public:
 	 * detachConsumer/detachProducer)
 	 */
 	bool running();
+
+	/**
+	 * called during crashes, to release the opened device (and other resources)
+	 * don't use this method for other purposes than emergencies
+	 */
+	void emergencyCleanup();
 };
 
 }
