@@ -7,7 +7,7 @@
     modify it under the terms of the GNU Library General Public
     License as published by the Free Software Foundation; either
     version 2 of the License, or (at your option) any later version.
-  
+
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
@@ -22,7 +22,6 @@
 
 #include "config.h"
 
-#ifdef HAVE_ARTS_GSL
 #include "virtualports.h"
 #include "startupmanager.h"
 #include "gslschedule.h"
@@ -235,25 +234,27 @@ void AudioPort::setFloatValue(float f)
 
 void AudioPort::connect(Port *psource)
 {
+	if (source) return; // Error, should not happen (See BR70028)
 	source = psource->audioPort();
 	assert(source);
 	addAutoDisconnect(psource);
 
+	source->parent->_connectionCountChanged = parent->_connectionCountChanged = true;
 	source->destcount++;
 	sourcemodule = source->parent;
 
+	// GSL connect
 	GslTrans *trans = gsl_trans_open();
 	gsl_trans_add(trans, gsl_job_connect(source->parent->gslModule,
 										 source->gslEngineChannel,
 										 parent->gslModule,
 										 gslEngineChannel));
 	gsl_trans_commit(trans);
-
-	source->parent->_connectionCountChanged = parent->_connectionCountChanged = true;
 }
 
 void AudioPort::disconnect(Port *psource)
 {
+	if (!source || source != psource->audioPort()) return; // Error, should not happen (See BR70028)
 	assert(source);
 	assert(source == psource->audioPort());
 	removeAutoDisconnect(psource);
@@ -474,13 +475,13 @@ void StdScheduleNode::rebuildConn()
 			ci != outConn[c]->autoDisconnect.end(); ci++)
 		{
 			AudioPort *dest = (*ci)->audioPort();
-
-			if (dest) {
+			if( dest )
+			{
 				gsl_trans_add(trans,
-						gsl_job_connect(outConn[c]->parent->gslModule,
-								outConn[c]->gslEngineChannel,
-								dest->parent->gslModule,
-								dest->gslEngineChannel));
+					gsl_job_connect(outConn[c]->parent->gslModule,
+									outConn[c]->gslEngineChannel,
+									dest->parent->gslModule,
+									dest->gslEngineChannel));
 			}
 			else
 			{
@@ -634,14 +635,14 @@ void StdScheduleNode::requireFlow()
 	gslMainLoop.run();
 }
 
-bool StdScheduleNode::suspendable()
+AutoSuspendState StdScheduleNode::suspendable()
 {
 	if(running) {
 		accessModule();
-		return (module->autoSuspend() != asNoSuspend);
+		return module->autoSuspend();
 	}
 	// if its not running, who cares?
-	return true;
+	return asSuspend;
 }
 
 void StdScheduleNode::suspend()
@@ -649,16 +650,16 @@ void StdScheduleNode::suspend()
 	if(running) {
 		accessModule();
 		suspended = true;
-		if(module->autoSuspend() == asSuspendStop) stop();
+		if((module->autoSuspend() & asSuspendMask) == asSuspendStop) stop();
 	}
 }
 
 void StdScheduleNode::restart()
 {
-	if(running == false && suspended == true) {
+	if(suspended) {
 		accessModule();
 		suspended = false;
-		if(module->autoSuspend() == asSuspendStop) start();
+		if(!running && (module->autoSuspend() & asSuspendMask) == asSuspendStop) start();
 	}
 }
 
@@ -725,6 +726,8 @@ void StdScheduleNode::connect(const string& port, ScheduleNode *dest,
 		return;
 	}
 
+	flowSystem->restart();
+
 	Port *p1 = findPort(port);
 	Port *p2 = ((StdScheduleNode *)dest)->findPort(destport);
 
@@ -752,6 +755,8 @@ void StdScheduleNode::disconnect(const string& port, ScheduleNode *dest,
 		return;
 	}
 
+	flowSystem->restart();
+
 	Port *p1 = findPort(port);
 	Port *p2 = ((StdScheduleNode *)dest)->findPort(destport);
 
@@ -838,7 +843,7 @@ StdFlowSystem::StdFlowSystem()
 		GslConfigValue values[2] = { { "wave_chunk_padding", 8 }, { 0, 0 } };
 		gsl_is_initialized = true;
 
-		g_thread_init(0);
+                if (!g_thread_supported ()) g_thread_init(0);
 		gsl_init(values, (GslMutexTable *)gslGlobalMutexTable);
 
 		/*
@@ -881,11 +886,107 @@ bool StdFlowSystem::suspended()
 
 bool StdFlowSystem::suspendable()
 {
+	/*
+	 * What it does:
+	 * -------------
+	 *
+	 * The suspension algorithm will first divide the graph of modules into
+	 * subgraphs of interconnected modules. A subgraph is suspendable if
+	 * all of its modules are suspendable and the subgraph does not contain
+	 * producer(s) and consumer(s) at the same time.
+	 *
+	 * Finally, our module graph is suspendable if all its subgraphs are.
+	 *
+	 * How it is implemented:
+	 * ----------------------
+	 *
+	 * For efficiency reasons, both steps are merged together. First all
+	 * modules will be marked as unseen. Then a module is picked and
+	 * all modules that it connects to are recursively added to the
+	 * subgraph.
+	 */
+
+	/*
+	 * initialization: no nodes are seen
+	 */
 	std::list<StdScheduleNode *>::iterator i;
-	for(i = nodes.begin();i != nodes.end();i++)
+	for (i = nodes.begin(); i != nodes.end(); i++)
 	{
 		StdScheduleNode *node = *i;
-		if(!node->suspendable()) return false;
+		node->suspendTag = false;
+	}
+
+	stack<StdScheduleNode*> todo;
+	for(i = nodes.begin(); i != nodes.end(); i++)
+	{
+		bool haveConsumer = false;
+		bool haveProducer = false;
+
+		/*
+		 * examine the subgraph consisting of all nodes connected to (*i)
+		 * (only will do anything if suspendTag is not already set)
+		 */
+
+		todo.push(*i);
+		do
+		{
+			StdScheduleNode *node = todo.top();
+			todo.pop();
+
+			if(!node->suspendTag)
+			{
+				node->suspendTag = true;	// never examine this node again
+
+				switch (node->suspendable())
+				{
+					case asNoSuspend|asProducer:
+					case asNoSuspend|asConsumer:
+					case asNoSuspend:
+							return false;
+						break;
+					case asSuspend:
+					case asSuspendStop:	
+							/* nothing */
+						break;
+					case asSuspend|asProducer:
+					case asSuspendStop|asProducer:
+							if(haveConsumer)
+								return false;
+							else
+								haveProducer = true;
+						break;
+					case asSuspend|asConsumer:
+					case asSuspendStop|asConsumer:
+							if(haveProducer)
+								return false;
+							else
+								haveConsumer = true;
+						break;
+					default:
+						arts_fatal("bad suspend value %d", node->suspendable());
+				}
+
+				unsigned int c;
+				for(c = 0; c < node->inConnCount; c++)
+				{
+					if(node->inConn[c]->source)
+						todo.push(node->inConn[c]->source->parent);
+				}
+	
+				for(c = 0; c < node->outConnCount; c++)
+				{
+					std::list<Port *>::iterator ci;
+
+					for(ci = node->outConn[c]->autoDisconnect.begin();
+						ci != node->outConn[c]->autoDisconnect.end(); ci++)
+					{
+						AudioPort *dest = (*ci)->audioPort();
+						if(dest)
+							todo.push(dest->parent);
+					}
+				}
+			}
+		} while(!todo.empty());
 	}
 	return true;
 }
@@ -1085,5 +1186,5 @@ public:
 	}
 } sfs;
 
-}
-#endif
+};
+
