@@ -173,7 +173,7 @@ public:
 
 	AutoSuspendState autoSuspend()
 	{
-		return asSuspendStop;
+		return static_cast<AutoSuspendState>(asSuspendStop|asConsumer);
 	}
 
 	void calculateBlock(unsigned long samples)
