@@ -86,6 +86,11 @@ public:
 		}
 	}
 
+	AutoSuspendState autoSuspend()
+	{
+		return static_cast<AutoSuspendState>(asSuspend|asProducer);
+	}
+
 	void calculateBlock(unsigned long samples)
 	{
 		// no audio subsystem, no play
