@@ -134,9 +134,10 @@ string MCOPUtils::getFullHostname()
 	string result;
 	struct hostent *hp;
 
-	if(gethostname(buffer,1024) == 0)
+	if(gethostname(buffer,1024) == 0) {
+		buffer[1023] = '\0';
 		result = buffer; 
-	else
+	} else
 		return "localhost";
 
 	/*
